package com.chinaitop.depot.system.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.chinaitop.depot.common.CommonObject;
import com.chinaitop.depot.system.model.UserInfo;
import com.chinaitop.depot.system.mapper.UserLoginLogMapper;
import com.chinaitop.depot.system.model.UserLoginLog;
import com.chinaitop.depot.system.model.UserLoginLogExample;
import com.chinaitop.depot.system.model.UserLoginLogVO;
import com.chinaitop.depot.system.param.UserLoginLogPageParam;
import com.chinaitop.depot.system.service.UserLoginLogService;
import com.chinaitop.depot.utils.DateUtils;
import com.chinaitop.depot.utils.IPUtil;
import com.chinaitop.depot.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * @author qingsong.han
 * @description:
 * @create 2024-10-31 15:38
 */
@Slf4j
@Service
public class UserLoginLogServiceImpl implements UserLoginLogService {
    @Resource
    private RedisUtil redisUtil;
    @Resource
    private IPUtil ipUtil;
    @Resource
    private UserLoginLogMapper userLoginLogMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveData(HttpServletRequest request, UserInfo userInfo, String loginType) {
        // 查询是否登录成功
        if (redisUtil.hasKey(userInfo.getUsername() + CommonObject.sessionId)) {
            UserLoginLog log = new UserLoginLog();
            log.setId(UUID.randomUUID().toString());
            log.setUserName(userInfo.getUsername());
            log.setRealName(userInfo.getRealName());
            log.setUserIp(ipUtil.getIpAddr(request));
            log.setInLoginTime(new Date());
            log.setSessionId(request.getSession().getId());
            log.setLoginType(loginType);
            userLoginLogMapper.insertSelective(log);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateData(HttpServletRequest request) {
        String sessionId = request.getSession().getId();
        if (StringUtils.hasText(sessionId)) { // 不为空
            // 更新
            this.updateDataTime(sessionId);
        }
    }

    private void updateDataTime(String sessionId) {
        // 更新
        UserLoginLogExample example = new UserLoginLogExample();
        UserLoginLogExample.Criteria criteria = example.createCriteria();
        criteria.andSessionIdEqualTo(sessionId);
        // 更新退出登录时间
        UserLoginLog loginLog = new UserLoginLog();
        loginLog.setOutLoginTime(new Date());
        userLoginLogMapper.updateByExampleSelective(loginLog, example);
    }

    @Override
    public void selectOutLoginIsNull() {
        // 查询未正常操作退出登录的(例如:直接关闭浏览器登录超时...)
        UserLoginLogExample example = new UserLoginLogExample();
        UserLoginLogExample.Criteria criteria = example.createCriteria();
        criteria.andOutLoginTimeIsNull();
        List<UserLoginLog> userLoginLogs = userLoginLogMapper.selectByExample(example);
        userLoginLogs.forEach(loginLog -> {
            // key不存在,已退出登录或已过期
            if (!redisUtil.hasKey(loginLog.getUserName() + CommonObject.sessionId)) {
                // 更新
                this.updateDataTime(loginLog.getSessionId());
            } else {
                // 获取session id
                String redisSessionId = redisUtil.get(loginLog.getUserName() + CommonObject.sessionId).toString();
                // 未更新session id
                String sessionId = loginLog.getSessionId();
                // 非正常退出后再次登录session id 不同,更新上次退出登录时间
                if (!sessionId.equals(redisSessionId)) {
                    this.updateDataTime(sessionId);
                }
            }
        });
    }

    @Override
    public List<UserLoginLog> getPageList(UserLoginLogPageParam param) {
        UserLoginLogExample example = new UserLoginLogExample();
        UserLoginLogExample.Criteria criteria = example.createCriteria();
        // 登录时间
        if (!ObjectUtils.isEmpty(param.getInLoginTimeStart()) && !ObjectUtils.isEmpty(param.getInLoginTimeEnd())) {
            criteria.andInLoginTimeBetween(param.getInLoginTimeStart(), param.getInLoginTimeEnd());
        }
        // 注销时间
        if (!ObjectUtils.isEmpty(param.getOutLoginTimeStart()) && !ObjectUtils.isEmpty(param.getOutLoginTimeEnd())) {
            criteria.andOutLoginTimeBetween(param.getOutLoginTimeStart(), param.getOutLoginTimeEnd());
        }
        if (StringUtils.hasText(param.getRealName())) {
            criteria.andRealNameLike(param.getRealName());
        }

        return userLoginLogMapper.selectByExample(example);
    }

    @Override
    public void export(String fileName, UserLoginLogPageParam param, HttpServletRequest request, HttpServletResponse response) {
        try {
            if (!StringUtils.hasText(fileName)) {
                fileName = LocalDateTime.now().toString();
            }

            // 设置响应信息，让浏览器下载文件
            fileName = URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");

            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");

            EasyExcel.write(response.getOutputStream()) // 或, GrainTempExcel.class
                    .head(UserLoginLogVO.class) // write 中, GrainTempExcel.class 二选一
                    .excelType(ExcelTypeEnum.XLSX)
                    .sheet("用户登录日志")
                    .doWrite(this.getData(param));

        } catch (IOException e) {
            log.error("摄像头管理导出异常：{}", e);
            throw new RuntimeException(e);
        }
    }

    private List<UserLoginLogVO> getData(UserLoginLogPageParam param) {
        List<UserLoginLog> pageList = this.getPageList(param);
        List<UserLoginLogVO> list = new ArrayList<>();
        pageList.forEach(userLoginLog -> {
            UserLoginLogVO logVO = new UserLoginLogVO();
            BeanUtils.copyProperties(userLoginLog, logVO);
            list.add(logVO);
        });
        return list;
    }

}
